
subroutine CalcEnergy(Pos, PosT ,M, L, rc, PEnergy, Dim, NAtom, Nchain)
    implicit none
    integer, intent(in) :: M, Dim, NAtom, Nchain
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in), dimension(0:Nchain-1, 0:Dim-1) :: PosT
    real(8), intent(in), dimension(0:Dim-1) :: L
    real(8), intent(in) :: rc
    real(8), intent(out) :: PEnergy
    real(8), dimension(0:Dim-1) :: rij, Posi
    real(8) :: d2, d, id2, id6, id12
    real(8) :: rc2, shift
    real(8), parameter :: k = 3000., r0 = 1.
    integer :: i, j
    !======== YOUR CODE HERE ========
    PEnergy = 0.
    !Caculating prefactors
    rc2 = rc*rc
    shift = -4.*(rc2**(-6.)-rc2**(-3.))
    !Main loop
    do i = 0, NAtom - 1
        !store Pos(i,:) in a temporary array for faster access in j loop
        Posi = Pos(i,:)
    	do j = i + 1, NAtom - 1
    	    rij = Pos(j,:) - Posi
            rij(0) = rij(0) - L(0) * dnint(rij(0)/L(0))
            rij(1) = rij(1) - L(1) * dnint(rij(1)/L(1))
            d2 = sum(rij * rij)
            if (j == i + 1 .and. mod(j, M) > 0) then
                !this is a bonded interaction;
                !use the harmonic bond potential here
                d = sqrt(d2)
                PEnergy = PEnergy + k*((d2 +r0*r0)/2. - d * r0)
            else
                !this is a nonbonded interaction;
                !use the Lennard-Jones potential here
                if (d2 > rc2) then
                    cycle
                end if
                id2 = 1. / d2            !inverse squared distance
                id6 = id2 * id2 * id2    !inverse sixth distance
                id12 = id6 * id6         !inverse twelvth distance
                PEnergy = PEnergy + 4. * (id12 - id6) + shift
            endif
        enddo
        ! Adding Unpairwised potential terms (tethering and wall potential)
        ! Wall potential
        PEnergy = PEnergy + 4.*((10.*Posi(2))**(-12.))
        ! Graftying term
        if (Mod(i,M)==0) then
            rij = PosT(i/M, :) - Posi
            rij(0) = rij(0) - L(0) * dnint(rij(0)/L(0))
            rij(1) = rij(1) - L(1) * dnint(rij(1)/L(1))
            d2 = sum(rij * rij)
            d = sqrt(d2)
            PEnergy = PEnergy + k*((d2 +r0*r0)/2. - d * r0)
        endif 
            
    enddo

    !======== YOUR CODE HERE ========
end subroutine


subroutine CalcEnergyForces(Pos, PosT , M, L, rc, PEnergy, Forces, Dim, NAtom, Nchain)
    implicit none
    integer, intent(in) :: M, Dim, NAtom, Nchain
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in), dimension(0:Nchain-1, 0:Dim-1) :: PosT
    real(8), intent(in), dimension(0:Dim-1) :: L
    real(8), intent(in) :: rc
    real(8), intent(out) :: PEnergy
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Forces
    real(8), dimension(0:Dim-1) :: rij, Fij, Posi
    real(8) :: d2, d, id2, id6, id12
!f2py intent(in, out, inplace) :: Forces
    real(8) :: rc2, shift
    real(8), parameter :: k = 3000., r0 = 1.
    integer :: i, j
    !======== YOUR CODE HERE ========
    PEnergy = 0.
    Forces = 0.
    !Caculating prefactors
    rc2 = rc* rc
    shift = -4.*(rc2**(-6.)-rc2**(-3.))
    !Main loop
    do i = 0, NAtom - 1
        !store Pos(i,:) in a temporary array for faster access in j loop
        Posi = Pos(i,:)
        ! Calculating pairwised potential and force
    	do j = i + 1, NAtom - 1
    	    rij = Pos(j,:) - Posi
            rij(0) = rij(0) - L(0) * dnint(rij(0)/L(0))
            rij(1) = rij(1) - L(1) * dnint(rij(1)/L(1))
            d2 = sum(rij * rij)
            if (j == i + 1 .and. mod(j, M) > 0) then
                !this is a bonded interaction;
                !use the harmonic bond potential here
                d = sqrt(d2)
                PEnergy = PEnergy + k*((d2 +r0*r0)/2. - d * r0)
                Fij = k* rij * (1.- r0/d)
                Forces(i,:) = Forces(i,:) + Fij
                Forces(j,:) = Forces(j,:) - Fij
            else
                !this is a nonbonded interaction;
                !use the Lennard-Jones potential here
                if (d2 > rc2) then
                    cycle
                end if
                id2 = 1. / d2            !inverse squared distance
                id6 = id2 * id2 * id2    !inverse sixth distance
                id12 = id6 * id6         !inverse twelvth distance
                PEnergy = PEnergy + 4. * (id12 - id6) + shift
                Fij = rij * ((-48. * id12 + 24. * id6) * id2)
                Forces(i,:) = Forces(i,:) + Fij
                Forces(j,:) = Forces(j,:) - Fij
            endif
        enddo
        ! Adding Unpairwised potential terms (tethering and wall potential)
        ! Wall potential
        PEnergy = PEnergy + 4.*((10.* Posi(2))**(-12.))
        Forces(i,2)= Forces(i,2) + 12. *4.* ((10.*Posi(2))**(-13.))
        ! Graftying term
        if (Mod(i,M)==0) then
            rij = PosT(i/M, :) - Posi
            rij(0) = rij(0) - L(0) * dnint(rij(0)/L(0))
            rij(1) = rij(1) - L(1) * dnint(rij(1)/L(1))
            d2 = sum(rij * rij)
            d = sqrt(d2)
            PEnergy = PEnergy + k*((d2 +r0*r0)/2. - d * r0)
            Fij = k* rij * (1.- r0/d)
            Forces(i,:) = Forces(i,:) + Fij
        endif 
    enddo
end subroutine


subroutine VVIntegrate(Pos, Vel, Accel, PosT, M, L, rc, dt, KEnergy, PEnergy, Dim, NAtom, Nchain)
    implicit none
    integer, intent(in) :: M, Dim, NAtom, Nchain
    real(8), intent(in) :: rc, dt
    real(8), intent(in), dimension(0:Dim-1) :: L
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Pos, Vel, Accel
    real(8), intent(in), dimension(0:Nchain-1, 0:Dim-1) :: PosT
!f2py intent(in,out,inplace) :: Pos, Vel, Accel
    real(8), intent(out) :: KEnergy, PEnergy
    integer :: k
    external :: CalcEnergyForces
    Pos = Pos + dt * Vel + 0.5 * dt*dt * Accel
    
    do k = 0, NAtom - 1
        if (Pos(k,2)<=0) then
            Pos(k,2) = 0.01
        elseif (Pos(k,2)>= L(2)) then
            Pos(k,2)= L(2) - 0.01
        endif
    enddo
    
    Vel = Vel + 0.5 * dt * Accel
    call CalcEnergyForces(Pos, PosT, M, L, rc, PEnergy, Accel, Dim, NAtom, Nchain)
    Vel = Vel + 0.5 * dt * Accel
    KEnergy = 0.5 * sum(Vel*Vel)
end subroutine
